(function() {
    this.mraid = {
  
  //--- costants
	VERSIONS : {
		V1  : '1.0',
		V2  : '2.0'
	},
	PLACEMENTS : {
		INLINE       : 'inline',
		INTERSTITIAL : 'interstitial'
	},
	ORIENTATIONS : {
		NONE      : 'none',
		PORTRAIT  : 'portrait',
		LANDSCAPE : 'landscape'
	},
  	CLOSEPOSITIONS : {
		TOPLEFT     : 'top-left',
		TOPRIGHT    : 'top-right',
		BOTTOMLEFT  : 'bottom-left',
		BOTTOMRIGHT : 'bottom-right',
		CENTER      : 'center'
	},
    STATES : {
 		LOADING		:'loading',
        DEFAULT     :'default',
        RESIZED     :'resized',
        EXPANDED    :'expanded',
        HIDDEN      :'hidden'
    },
    CONTROLS : {
        BACK    :'back',
        FORWARD :'forward',
        REFRESH :'refresh',
        ALL     :'all'
    },
    EVENTS : {
        ASSETREADY          :'assetReady',
        ASSETREMOVED        :'assetRemoved',
        ASSETRETIRED        :'assetRetired',
        ERROR               :'error',
        INFO                :'info',
        HEADINGCHANGE       :'headingChange',
        KEYBOARDCHANGE      :'keyboardChange',
        LOCATIONCHANGE      :'locationChange',
        NETWORKCHANGE       :'networkChange',
        ORIENTATIONCHANGE   :'orientationChange',
        RESPONSE            :'response',
        SCREENCHANGE        :'screenChange',
        SHAKE               :'shake',
        SIZECHANGE          :'sizeChange',
        STATECHANGE         :'stateChange',
        TILTCHANGE          :'tiltChange',
        VIEWABLECHANGE      :'viewableChange',
		READY				:'ready',
		CALENDAREVENTADDED  :'calendarEventAdded',
		PICTUREADDED        :'pictureAdded'
    },
    FEATURES : {
        SMS         :'sms',
        PHONE       :'phone',
        CALENDAR    :'calendar',
        STOREPICTURE:'storePicture',
		INLINEVIDEO	:'inlineVideo'
    },
    NETWORK : {
        OFFLINE :'offline',
        WIFI    :'wifi',
        CELL    :'cell',
        UNKNOWN :'unknown'
    },

  	//--- supported methods, alphabetical list
    addEventListener: function(event, listener) {
      if (event === "ready" || event === "stateChange" || event === "viewableChange" || event === "error" || event === "sizeChange") {
        mraid_bridge.logMessage("adding event listener for " + event);
        return (listeners[event] || (listeners[event] = [])).push(listener);
      }else{
        mraid_bridge.logMessage("no event listener for " + event);
      }
    },
    //If the attempt to create the calendar event fails or is cancelled by the user, it will trigger an error.
    createCalendarEvent: function(date, title, body){
		title = trim(title);
		body = trim(body);
        mraid_bridge.logMessage("calendar event " + date + "/" + title + "/" + body);
      	//mraid_bridge.alertMessage("calendar event " + date + "/" + title + "/" + body);
        if (!(mraid.support['calendar'])) {
      		mraid_bridge.alertMessage("Method not supported by this client.");
        	mraid_bridge.logMessage("Method not supported by this client.");
        	mraid.fireErrorEvent('Method not supported by this client.', 'createEvent');
        } else if (!date || typeof date != 'object' || !date.getDate) {
        	mraid.fireErrorEvent('Valid date required.', 'createEvent');
        } else if (!title || typeof title != 'string' || title.length == 0) {
        	mraid.fireErrorEvent('Valid title required.', 'createEvent');
		}	else if (!body || typeof body != 'string' || body.length == 0) {
        	mraid.fireErrorEvent('Valid body required.', 'createEvent');
        } else {
      		var msecs=(date.getTime()-date.getMilliseconds());
        	mraid_bridge.createEvent(msecs, title, body);
        }    
    },
    close: function() {
      mraid_bridge.close();
    },
    expand: function() {
      var url;
      url = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      mraid_bridge.logMessage("in expand : " + state);
      if ((url != null ? url.length : void 0) === 0) {
        return mraid_bridge.expand();
      } else {
        return mraid_bridge.expand(url[0]);
      }
    },
    fireErrorEvent: function(message, action) {
      var listener, _i, _len, _ref, _results;
      _ref = listeners["error"];
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        _results.push(listener(message, action));
      }
      return _results;
    },
    fireEvent: function(event) {
      var listener, _i, _len, _ref, _results;
      mraid_bridge.logMessage("fireEvent : " + event);
      if (listeners[event]) {
        _ref = listeners[event];
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          listener = _ref[_i];
          if (event === "ready") listener();
          if (event === "stateChange") listener(state);
          if (event === "viewableChange") {
            _results.push(listener(viewable));
          } else {
            _results.push(void 0);
          }
        }
        return _results;
      }
    },
    getCurrentPosition: function(){
	    mraid_bridge.logMessage("getCurrentPosition: " + stringify(currentPosition));
	    mraid_bridge.alertMessage("getCurrentPosition: " + stringify(currentPosition));
    	return stringify(currentPosition);
    },
    getDefaultPosition: function(){
	    mraid_bridge.logMessage("getDefaultPosition: " + stringify(defaultPosition));
	    mraid_bridge.alertMessage("getDefaultPosition: " + stringify(defaultPosition));
    	return stringify(defaultPosition);
    },
    getExpandProperties: function() {
      mraid_bridge.logMessage("getExpandProperties: " + stringify(expandProperties));
      mraid_bridge.alertMessage("getExpandProperties: " + stringify(expandProperties));
      return stringify(expandProperties);
    },
    getMaxSize: function(){
	    mraid_bridge.logMessage("getMaxSize: " + stringify(maxSize));
	    mraid_bridge.alertMessage("getMaxSize: " + stringify(maxSize));
    	return stringify(maxSize);
    },
    getPlacementType: function() {
      mraid_bridge.logMessage("getPlacementType: " + placementType);
      mraid_bridge.alertMessage("getPlacementType: " + placementType);
      return placementType;
    },
    getResizeProperties: function() {
      mraid_bridge.logMessage("getResizeProperties: " + stringify(resizeProperties));
      mraid_bridge.alertMessage("getResizeProperties: " + stringify(resizeProperties));
      return stringify(resizeProperties);
    },
    getScreenSize: function(){
	    mraid_bridge.logMessage("getScreenSize: " + stringify(screenSize));
	    mraid_bridge.alertMessage("getScreenSize: " + stringify(screenSize));
    	return stringify(screenSize);
    },
    getSize: function(){
	    mraid_bridge.logMessage("getSize: " + stringify(size));
	    mraid_bridge.alertMessage("getSize: " + stringify(size));
    	return stringify(size);
    },
    getState: function() {
      mraid_bridge.logMessage("getState: " + state);
      mraid_bridge.alertMessage("getState: " + state);
      return state;
    },
    getSupports: function(){
    	var features = mraid_bridge.getSupports();
      mraid_bridge.logMessage("getSupports: " + features);
      mraid_bridge.alertMessage("getSupports: " + features);
      return features;
    },
    getVersion: function() {
      mraid_bridge.logMessage("getVersion: " + mraidVersion);
      mraid_bridge.alertMessage("getVersion: " + mraidVersion);
      return mraidVersion;
    },
    isViewable: function() {
      mraid_bridge.logMessage("isViewable: " + viewable);
      mraid_bridge.alertMessage("isViewable: " + viewable);
      return viewable;
    },
    notAvailable: function(){
      mraid_bridge.logMessage("function not available in MRAID 1.0");
      mraid_bridge.alertMessage("function not available in MRAID 1.0");
    },
    open: function(url) {
      mraid_bridge.logMessage("open clicked");
      mraid_bridge.open(url);
    },
    //play a video on the device via the device's native, external player.
    //To play video inline (on devices where that feature is supported), use HTML5 video tags.
    playVideo: function(url){
      mraid_bridge.logMessage("try to playing:"+url);
      //mraid_bridge.alertMessage("try to playing:"+url);
        if (!url || typeof url != 'string') {
      		mraid_bridge.alertMessage("url not valid");
        	mraid_bridge.logMessage("url not valid");
        } else {
            mraid_bridge.playVideo(url);
        }
    },
    removeEventListener: function() {
      var event, l, listener;
      event = arguments[0], listener = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (listeners[event] && listener.length > 0) {
        return listeners[event] = (function() {
          var _i, _len, _ref, _results;
          _ref = listeners[event];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            l = _ref[_i];
            if (l !== listener[0]) _results.push(l);
          }
          return _results;
        })();
      } else {
        return delete listeners[event];
      }
    },
    resize: function() {
      mraid_bridge.resize();
    },
    setCurrentPosition: function(properties) {
      currentPosition.x = properties.x;
      currentPosition.y = properties.y;
      currentPosition.width = properties.width;
      currentPosition.height = properties.height;
    },
    setDefaultPosition: function(properties) {
      defaultPosition.x = properties.x;
      defaultPosition.y = properties.y;
      defaultPosition.width = properties.width;
      defaultPosition.height = properties.height;
    },
    setExpandProperties: function(properties) {
      if (properties.width) expandProperties.width = properties.width;
      if (properties.height) expandProperties.height = properties.height;
      if (properties.useCustomClose) {
        expandProperties.useCustomClose = properties.useCustomClose;
      }
      return mraid_bridge.setExpandProperties(JSON.stringify(expandProperties));
    },
    setMaxSize: function(properties) {
      maxSize.width = properties.width;
      maxSize.height = properties.height;
    },
    setPlacementType: function(type) {
      if (type === 0) {
        return placementType = "inline";
      } else if (type === 1) {
        return placementType = "interstitial";
      }
    },
    setResizeProperties: function(properties) {
      if (properties.width) resizeProperties.width = properties.width;
      if (properties.height) resizeProperties.height = properties.height;
      return mraid_bridge.setResizeProperties(JSON.stringify(resizeProperties));
    },
    setSize: function(properties) {
      size.width = properties.width;
      size.height = properties.height;
    },
    setState: function(state_id) {
      switch (state_id) {
        case 1:
          state = "loading";
          break;
        case 2:
          state = "hidden";
          break;
        case 3:
          state = "default";
          break;
        case 4:
          state = "expanded";
          break;
        case 5:
          state = "resized";
      }
      mraid_bridge.logMessage("in setState : " + state);
      return mraid.fireEvent("stateChange");
    },
    setViewable: function(is_viewable) {
      viewable = is_viewable;
      return mraid.fireEvent("viewableChange");
    },
    storePicture: function(url){
    //If the attempt to add the picture fails for any reason or is cancelled by the user, it will trigger an error.
      mraid_bridge.logMessage("try to storing:"+url);
      //mraid_bridge.alertMessage("try to storing:"+url);
		//var r=confirm("Store picture at url:"+url+"?");
		//if (r==true){
      		mraid_bridge.storePicture(url);
		//}else{
		//	mraid.fireErrorEvent("Storing deleted!", "storePicture");
		//}
    },
    support: function(feature) {
        if (supports[feature]) return true;
        else return false;
    },
    useCustomClose: function(useCustomCloseParams) {
      expandProperties.useCustomClose = useCustomCloseParams;
      return mraid_bridge.setExpandProperties(JSON.stringify(expandProperties));
    }  
  };
  
  //object variable, private properties
  	//MRAID version supported
    var mraidVersion = mraid.VERSIONS.V2;
    
    var state = mraid.STATES.LOADING;
    var placementType = mraid.PLACEMENTS.INLINE;
    
    var size = {
        width:0,
        height:0
    };
    
    var screenSize = {
        width:screen.width,
        height:screen.height
    };
    
    var defaultPosition = {
        x:0,
        y:0,
        width:0,
        height:0
    };
    
    var currentPosition = {
        x:0,
        y:0,
        width:0,
        height:0
    };
    //default maxSize is screen dimensions
    var maxSize = {
        width:screen.width,
        height:screen.height
    };
    
    //with MRAID2.0 isModal is always true
    var expandProperties = {
		width:0,
		height:0,
		useCustomClose:false,
		isModal:true,
		allowOrientationChange: true,
		forceOrientation: mraid.ORIENTATIONS.NONE
    };
    //if allowOffscreen is true can go off screen to resize ad
    //if it is false, ad remains on screen (SDK reposition ad)
	var resizeProperties = {
		width: 0,
		height: 0,
		customClosePosition: mraid.CLOSEPOSITIONS.TOPRIGHT,
		offsetX: 0,
		offsetY: 0,
		allowOffscreen: true
	};
    //
    
    var supports = {
        'sms':true,
        'phone':true,
        'calendar':true,
        'storePicture':true,
		'inlineVideo':true
    };
    
    var heading = -1;
    var keyboardState = false;
    var location = null;
    var network = mraid.NETWORK.UNKNOWN;
    var orientation = -1;
    var shakeProperties = null;
    var tilt = null;
    var assets = {};
    var cacheRemaining = -1;
    var viewable = false;
    
	//support variable
  	var listeners = {};
  	var __slice = Array.prototype.slice;
  	
  	function trim(str) {
		return str.replace(/^\s+|\s+$/g,"");
	}
  	
}).call(this);
